/*
 Mini Temp Logger - Console Interface
 Version: v1.0
 Author: Alex from insideGadgets (www.insidegadgets.com)
 Created: 22/02/2015
 Last Modified: 22/03/2017
 
 Plug in the USB and run this program to change settings, RTC time and more.
 
 Based on V-USB example code by Christian Starkjohann and AVR ATtiny USB Tutorial at http://codeandlife.com
 
 */

#include <stdio.h>
#include <stdlib.h>
#include <stdint.h>
#include <string.h>
#include <time.h>
#include "setup.c"

int main(int argc, char **argv) {
	
	if (argc < 2) {
		printf("Mini Temp Logger Console Interface v1.0 by insideGadgets\n"\
				 "--------------------------------------------------------\n"\
				 "Used to change the settings, set the real time clock time and more.\n");
	}
	
	handle = usbOpenDevice(0x16C0, "insidegadgets.com", 0x05DC, "MTL");
	if (handle == NULL) {
		if (argc < 2) {
			fprintf(stderr, "Could not find USB device!\n");
		}
		else {
			printf("0");
		}
		exit(1);
	}
	
	check_arguments(argc, argv);
	
	int inLoop = true;
	while (inLoop == true) {
		printf("\nPlease select an option:\n"\
				 "1. Program delay time\n"\
				 "2. Read delay time\n"\
				 "3. Program logging function\n"\
				 "4. Read logging function\n"\
				 "5. RTC functions\n"\
				 "6. Read TMP102 temperature\n"\
				 "7. Read voltage\n"
				 "0. Exit\n");
		int optionSelected = read_number_w_min_max(0, 7);
		
		if (optionSelected == 1) { // Program delay time
			printf("\n--- Programming delay time ---\n");
			
			input_dhms(); // Read days, hours, minutes, seconds
			printf("\nProgramming delay time as %i days, %i hours, %i minutes, %i seconds... ", days, hours, minutes, seconds);
			
			// Send to USB device
			clear_buffer();
			buffer[1] = seconds;
			buffer[2] = minutes;
			buffer[3] = hours;
			buffer[4] = days;
			usb_send_data(PROGRAM_DELAY_TIME, 4);
			verify_data_written(USB_DELAY_TIME, 4);
		}
		
		// Read delay time
		else if (optionSelected == 2) {
			printf("\n--- Read delay time ---\n");
			usb_read_data(USB_DELAY_TIME);
			print_interval_time_from_buffer();
		}
		
		// Program function to use
		else if (optionSelected == 3) {
			printf("\n--- Program logging function ---\n");
			printf("Please select 1 to log temperature and 2 to log voltage: ");
			
			int functionSelected = read_number_w_min_max(1, 2);
			
			printf("Programming function is use as ");
			if (functionSelected == 1) {
				printf("temperature logging... ");
			}
			else {
				printf("voltage logging... ");
			}
			
			// Send to USB device
			clear_buffer();
			buffer[1] = functionSelected;
			usb_send_data(PROGRAM_LOG_FUNCTION, 1);
			verify_data_written(USB_FUNCTION_SELECTED, 1);
		}
		
		// Read function to use
		else if (optionSelected == 4) {
			printf("\n--- Read logging function ---\n");
			usb_read_data(USB_FUNCTION_SELECTED);
			
			uint8_t readFunctionSelected = buffer[0];
			if (readFunctionSelected == 1) {
				printf("Function in use is temperature logging\n\n");
			}
			else {
				printf("Function in use is voltage logging\n\n");
			}
		}
		
		// RTC functions
		else if (optionSelected == 5) {
			int inRtcLoop = true;
			while (inRtcLoop == true) {
				
				printf("\n--- RTC Functions ---\n"\
						"Please select an RTC function option:\n"\
						"1. Program RTC time\n"\
						"2. Read current RTC time\n"\
						"3. Read logging start RTC time\n"\
						"4. Program RTC trim\n"\
						"5. Read RTC trim\n"\
						"6. Program RTC auto trim\n"\
						"7. Read RTC auto trim setting\n"\
						"8. Read RTC auto trim PPM and applied PPM\n"\
						"9. Program RTC auto trim seconds interval\n"\
						"10. Read RTC auto trim seconds interval\n"\
						"0. Go back\n");
				int rtcOptionSelected = read_number_w_min_max(0, 10);
				
				// Program RTC time
				if (rtcOptionSelected == 1) { 
					printf("\n--- Programming RTC time ---\n"\
							 "1. Read time from PC\n"\
							 "2. Program time manually\n");
					int timeOptionSelected = read_number_w_min_max(1, 2);
					
					// Get the date/time from the PC
					if (timeOptionSelected == 1) {
						clear_buffer();
						
						time_t rawtime;
						struct tm* timeinfo;
						char timebuffer[80];
						
						time (&rawtime);
						timeinfo = localtime (&rawtime);
						strftime (timebuffer,80,"%Y-%m-%d-%H-%M-%S",timeinfo);
						
						int tokenCount = 7;
						char *token;
						token = strtok(timebuffer, "-");
						while(token != NULL) {
							if (tokenCount == 7) { // Year
								buffer[tokenCount] = atoi(token) - 2000;
							}
							else {
								buffer[tokenCount] = atoi(token);
							}
							
							token = strtok(NULL, "-");
							tokenCount--;
						}
						
						// Convert to 12 hour format
						buffer[1] = 0; // Am
						if (buffer[4] >= 13) {
							buffer[4] -= 12;
							buffer[1] = 1; // Pm
						}
						else if (buffer[4] == 12) {
							buffer[1] = 1; // Pm
						}
						else if (buffer[4] == 0) {
							buffer[4] = 12;
						}
						
						// Copy to local variables
						uint8_t rtcTimeAmPm = buffer[1];
						uint8_t rtcTimeSeconds = buffer[2];
						uint8_t rtcTimeMinutes = buffer[3];
						uint8_t rtcTimeHour = buffer[4];
						uint8_t rtcTimeDate = buffer[5];
						uint8_t rtcTimeMonth = buffer[6];
						uint8_t rtcTimeYear = buffer[7];
						
						printf("\nProgramming RTC time as 20%i year, %i month, %i date, %i hour, %i minutes, %i seconds ", rtcTimeYear, rtcTimeMonth, rtcTimeDate, rtcTimeHour, rtcTimeMinutes, rtcTimeSeconds);
						if (rtcTimeAmPm == 0) {
							printf("AM... ");
						}
						else {
							printf("PM... ");
						}
						
						usb_send_data(PROGRAM_RTC_TIME, 7);
						verify_data_written(USB_RTC_CURRENT_TIME, 7);
					}
					else if (timeOptionSelected == 2) { // Enter in time manually
						printf("Please enter in the year (20xx), only the xx part: ");
						int rtcTimeYear = read_number_w_min_max(0, 99);
						
						printf("Please enter in the month: ");
						int rtcTimeMonth = read_number_w_min_max(1, 12);
						
						printf("Please enter in the date: ");
						int rtcTimeDate = read_number_w_min_max(1, 31);
						
						printf("Please enter in the hour (in 12 hr format): ");
						int rtcTimeHour = read_number_w_min_max(0, 12);
						
						printf("Please enter in the minutes:");
						int rtcTimeMinutes = read_number_w_min_max(0, 60);
						
						printf("Please enter in the seconds:");
						int rtcTimeSeconds = read_number_w_min_max(0, 60);
						
						printf("Please enter in the hour AM or PM\n"\
								 "0. AM\n"\
								 "1. PM\n");
						int rtcTimeAmPm = read_number_w_min_max(0, 1);
						
						printf("\nProgramming RTC time as 20%i year, %i month, %i date, %i hour, %i minutes, %i seconds ", rtcTimeYear, rtcTimeMonth, rtcTimeDate, rtcTimeHour, rtcTimeMinutes, rtcTimeSeconds);
						if (rtcTimeAmPm == 0) {
							printf("AM... ");
						}
						else {
							printf("PM... ");
						}
						
						clear_buffer();
						buffer[1] = rtcTimeAmPm;
						buffer[2] = rtcTimeSeconds;
						buffer[3] = rtcTimeMinutes;
						buffer[4] = rtcTimeHour;
						buffer[5] = rtcTimeDate;
						buffer[6] = rtcTimeMonth;
						buffer[7] = rtcTimeYear;
						usb_send_data(PROGRAM_RTC_TIME, 7);
						verify_data_written(USB_RTC_CURRENT_TIME, 7);
					}
				}
				
				// Read current RTC time
				else if (rtcOptionSelected == 2) {
					printf("\n--- Read current RTC time ---\n");
					usb_read_data(USB_RTC_CURRENT_TIME);
					print_time_from_buffer();
				}
				
				// Read the RTC time when the logging started
				else if (rtcOptionSelected == 3) {
					printf("\n--- Read RTC time when logging started ---\n");
					usb_read_data(USB_RTC_START_LOG_TIME);
					print_time_from_buffer();
				}
				
				// Program RTC trim
				else if (rtcOptionSelected == 4) { 
					printf("\n--- Program RTC trim ---\n");
					
					printf("Please enter the Trim sign bit (0 reduce clocks, 1 add clocks): ");
					int rtcTrimSign = read_number_w_min_max(0, 1);
					
					printf("Please enter the Trim (0 to 127): ");
					int rtcTrim = read_number_w_min_max(0, 127);
					
					if (rtcTrimSign == 0) {
						printf("\nProgramming Trim as -%i PPM (%i clock cycles)... ", rtcTrim, rtcTrim*2);
					}
					else {
						printf("\nProgramming Trim as +%i PPM (%i clock cycles)... ", rtcTrim, rtcTrim*2);
					}
					
					rtcTrim |= (rtcTrimSign << 7);
					
					clear_buffer();
					buffer[1] = rtcTrim;
					usb_send_data(PROGRAM_RTC_TRIM, 1);
					verify_data_written(USB_RTC_TRIM, 1);
				}
				
				// Read RTC Trim
				else if (rtcOptionSelected == 5) {
					printf("\n--- Read RTC trim ---\n");
					usb_read_data(USB_RTC_TRIM);
					
					uint8_t readRtcTrim = buffer[0];
					if (readRtcTrim & 1<<7) {
						printf("RTC Trim: +%i PPM (%i clock cycles)\n\n", readRtcTrim & 0x7F, (readRtcTrim & 0x7F) * 2);
					}
					else {
						printf("RTC Trim: -%i PPM (%i clock cycles)\n\n", readRtcTrim & 0x7F, (readRtcTrim & 0x7F) * 2);
					}
				}
				
				// Program RTC auto trim
				else if (rtcOptionSelected == 6) {
					printf("\n--- Program RTC auto trim ---\n");
					printf("Enter 0 to disable auto trim or 1 to enable auto trim (default): ");
					int rtcAutoTrim = read_number_w_min_max(0, 1);
					
					printf("Programming RTC auto trim as ");
					if (rtcAutoTrim == 1) {
						printf("on... ");
					}
					else {
						printf("off... ");
					}
					
					clear_buffer();
					buffer[1] = rtcAutoTrim;
					usb_send_data(PROGRAM_AUTOTRIM, 1);
					verify_data_written(USB_AUTOTRIM, 1);
				}
				
				// Read RTC Auto Trim
				else if (rtcOptionSelected == 7) {
					printf("\n--- Read RTC auto trim setting ---\n");
					usb_read_data(USB_AUTOTRIM);
					
					uint8_t readRtcAutoTrim = buffer[0];
					if (readRtcAutoTrim == 1) {
						printf("RTC Auto Trim is on\n\n");
					}
					else {
						printf("RTC Auto Trim is off\n\n");
					}
				}
				
				// Read RTC auto trim & calculated value
				else if (rtcOptionSelected == 8) {
					printf("\n--- Read RTC auto trim PPM and applied PPM  ---\n");
					usb_read_data(USB_AUTOTRIM_VAL_AND_CALC);					
					
					uint8_t readRtcAutoTrimVal = buffer[0];
					printf("RTC auto trim value: -%i PPM\n", readRtcAutoTrimVal);
					
					uint8_t readRtcAutoTrimCalc = buffer[1];
					if (readRtcAutoTrimCalc & 1<<7) {
						printf("When applied to static trim: +%i PPM (%i clock cycles)\n\n", readRtcAutoTrimCalc & 0x7F, (readRtcAutoTrimCalc & 0x7F) * 2);
					}
					else {
						printf("When applied to static trim: -%i PPM (%i clock cycles)\n\n", readRtcAutoTrimCalc & 0x7F, (readRtcAutoTrimCalc & 0x7F) * 2);
					}
				}
				
				// Program RTC auto trim seconds interval
				else if (rtcOptionSelected == 9) {
					printf("\n--- Program RTC auto trim interval ---\n");
					
					input_dhms(); // Read days, hours, minutes, seconds
					printf("\nProgramming auto trim interval time as %i days, %i hours, %i minutes, %i seconds... ", days, hours, minutes, seconds);
					
					clear_buffer();
					buffer[1] = seconds;
					buffer[2] = minutes;
					buffer[3] = hours;
					buffer[4] = days;
					usb_send_data(PROGRAM_AUTOTRIM_INTERVAL, 4);
					verify_data_written(USB_AUTOTRIM_INTERVAL, 4);
				}
				
				// Read RTC auto trim seconds interval
				else if (rtcOptionSelected == 10) {
					printf("\n--- Read RTC auto trim interval---\n");
					usb_read_data(USB_AUTOTRIM_INTERVAL);	
					print_interval_time_from_buffer();				
				}
				
				else if (rtcOptionSelected == 0) { // Exit
					inRtcLoop = false;
				}
			}
		}
		
		// Read TMP102 temperature
		else if (optionSelected == 6) {
			printf("\n--- Read TMP102 temperature ---\n");
			usb_read_data(USB_TEMP);	
			
			printf("Current temperature: ");
			int combinedValue = ((uint8_t) buffer[0] << 8) | (uint8_t) buffer[1];
			if (combinedValue >= 0x1C90 && combinedValue <= 0x1FFF) { // Convert negative temps
				combinedValue &= 0x1FF;
				combinedValue--;
				combinedValue ^= 0x1FF;
				printf("-");
			}
			printf("%4.2f\n", combinedValue * 0.0625);
		}
		
		// Read voltage
		else if (optionSelected == 7) {
			printf("\n--- Read voltage ---\n");
			usb_read_data(USB_VOLTAGE);	
			
			printf("Current voltage: ");
			int combinedValue = ((uint8_t) buffer[0] << 8) | (uint8_t) buffer[1];
			printf("%2.3f\n", (double) combinedValue * (0.00107421875 * 21) - (combinedValue * 0.00021484375)); // 0.000214 is the calculated difference between the actual ADC result and what the battery really measures
		}
		else if (optionSelected == 0) { // Exit
			inLoop = false;
		}
		else {
			printf("\nUnknown command\n\n");
		}
	}
	
	usb_close(handle);
	
	return 0;
}
