/*
 Mini Temp Logger
 Version: 1.0
 Author: Alex from insideGadgets (www.insidegadgets.com)
 Created: 22/02/2015
 Last Modified: 22/03/2017
 
 Used to log temperature using the TMP102 sensor every x seconds/minutes/hours/days which is saved to an external 
 1 Mbit I2C EEPROM. Can do voltage logging as well. You can change the setting via the interface program and
 transfer the data to your computer using the transfer program.
 
 Remember to change your fuse bits so the ATtiny84 uses 12MHz crystal and divide clock by 8
 avrdude -p ATtiny84 -c usbasp -U lfuse:w:0x4f:m -U hfuse:w:0xdf:m -U efuse:w:0xff:m
 
 */

// ATtiny24/44/84 Pin map
//
//                            +-\/-+
//                      VCC  1|o   |14 GND
//                 XTAL PB0  2|    |13 PA0 USB D+ 
//                 XTAL PB1  3|    |12 PA1 USB D-
//                Reset PB3  4|    |11 PA2 3.3V LDO Sense
//               USB D+ PB2  5|    |10 PA3 RTC 1Hz Sense
//                  LED PA7  6|    |9  PA4 SDA
// Button/Voltage Sense PA6  7|    |8  PA5 SCL
//                            +----+

#define F_CPU 12000000 // If this is changed, modify it in usbconfig.h too

#include <avr/io.h>
#include <avr/wdt.h>
#include <avr/eeprom.h>
#include <avr/interrupt.h>
#include <avr/sleep.h>
#include <util/delay.h>
#include <string.h>
#include "usbdrv/usbdrv.h"
#include "usbdrv/oddebug.h"
#include "usbconfig.h"
#include "setup.c" // See defines, variables, constants, functions here

int main (void) {
	setup();
	
	while (1) {
		// USB plugged in - transfer data
		if (PINA & (1<<usbSensePin)) {
			// Disable pin change interrupt
			cbi(GIMSK, PCIE0);
			
			// Set ADC prescale factor to 64
			// 12 MHz / 64 = 187 KHz, inside the desired 50-200 KHz range.
			sbi(ADCSRA, ADPS2);
			sbi(ADCSRA, ADPS1);
			
			// Enable ADC conversions and turn on ADC interrupt
			sbi(ADCSRA, ADEN);
			sbi(ADCSRA, ADIE);
			
			// Reset data count and set I2C delay cycles to USB mode
			dataCount = 0;
			I2C_DELAY_CYCLES = I2C_DELAY_CYCLES_USB;
			
			// Increase clock speed to 12MHz
			CLKPR = (1<<CLKPCE); // Prescaler enable
			CLKPR = 0; // Clock division factor 1 (0000)
			
			startUSB();
			inUSB = true;
			
			// Turn on watchdog to reset in 250ms if we don't pat the dog in time
			MCUSR &= ~(1<<WDRF);
			WDTCSR |= (1<<WDCE) | (1<<WDE);
			WDTCSR = (1<<WDE) | (1<<WDP2);
			
			PORTA |= (1<<ledPin);
			while (1) { // Keep looping until watchdog reset occurs when USB is unplugged
				usbPoll();
				
				if (PINA & (1<<usbSensePin)) {
					wdt_reset(); // Reset watchdog as still plugged in
				}
			}
		}
		
		// Start logging
		else if (logging == true) {
			PORTA &= ~(1<<ledPin);
			
			// Disable pin change on 3.3V LDO sense and enable RTC 1Hz sense
			cbi(PCMSK0, PCINT2);
			sbi(PCMSK0, PCINT3);
			
			// Reset data count and EEPROM variables and set I2C delay cycles to battery mode
			dataCount = 0;
			eepromBufferCounter = 0;
			eepromPageCounter = 0;
			eepromPageByteCounter = 0;
			I2C_DELAY_CYCLES = I2C_DELAY_CYCLES_BAT;
			
			// Copy RTC time to ATtiny EEPROM so we know the start date/time
			save_rtc_time_to_eeprom();
			
			// Set clock output at 1Hz
			soft_i2c_master_start_with_8bit_address(RTC_ADDRESS, RTC_CONTROL, NOREPEATSTART);
			soft_i2c_master_write(RTC_CONTROL_SQWEN_BIT);
			soft_i2c_master_stop();
			
			// Start the RTC (if it's not already started, this means the RTC stopped or RTC time hasn't been programmed)
			soft_i2c_master_start_with_8bit_address(RTC_ADDRESS, RTC_RTCSEC, REPEATSTART);
			uint8_t isRtcRunning = soft_i2c_master_read(1);
			soft_i2c_master_stop();
			if (!(isRtcRunning & RTC_RTCSEC_ST_BIT)) {
				soft_i2c_master_start_with_8bit_address(RTC_ADDRESS, RTC_RTCSEC, NOREPEATSTART);
				soft_i2c_master_write(RTC_RTCSEC_ST_BIT);
				soft_i2c_master_stop();
			}
			
			// Write first blank page on EEPROM
			soft_i2c_eeprom_page_blank(EEPROM_ADDR, eepromPageCounter);
			
			// Read delay time
			uint8_t delaySeconds = eeprom_read_byte((uint8_t*) delayTimeLocation);
			uint8_t delayMinutes = eeprom_read_byte((uint8_t*) delayTimeLocation+1);
			uint8_t delayHours = eeprom_read_byte((uint8_t*) delayTimeLocation+2);
			uint8_t delayDays = eeprom_read_byte((uint8_t*) delayTimeLocation+3);
			
			// Unprogrammed delay time?
			if (delaySeconds == blankEeprombyte) {
				delaySeconds = 0;
				delayMinutes = 5; // Set 5 minutes as the default delay time
				delayHours = 0;
				delayDays = 0;
			}
			
			// Read and calculate RTC auto trim interval time
			uint8_t autoTrimIntervalSeconds = eeprom_read_byte((uint8_t*) autoTrimIntervalLocation);
			uint8_t autoTrimIntervalMinutes = eeprom_read_byte((uint8_t*) autoTrimIntervalLocation+1);
			uint8_t autoTrimIntervalHours = eeprom_read_byte((uint8_t*) autoTrimIntervalLocation+2);
			uint8_t autoTrimIntervalDays = eeprom_read_byte((uint8_t*) autoTrimIntervalLocation+3);
			uint32_t autoTrimInterval = autoTrimIntervalSeconds + (autoTrimIntervalMinutes * 60) + (autoTrimIntervalHours * 3600) + (autoTrimIntervalDays * 86400);
			uint32_t autoTrimIntervalCounter = autoTrimInterval;
			
			while (logging == true) {
				cbi(PCMSK0, PCINT6); // Disable button interrupt so this can't be interrupted
				PORTA |= (1<<ledPin); // Turn on LED (which turns on mosfet too for voltage logging)
				
				int loggingResult = 0;
				if (functionSelect == LOG_TEMPERATURE) { // Temp logging
					loggingResult = tmp102_read_temp(TEMP_ADDR); // Read TMP102
				}
				else { // Voltage logging
					watchdog_sleep(T16MS); // Wait a little for LED to light up
					
					// Enable ADC conversions and turn on ADC interrupt
					sbi(ADCSRA, ADEN);
					sbi(ADCSRA, ADIE);
					_delay_us(2);
					
					adc_read(adcPin); // Ignore ADC result on first conversion
					loggingResult = adc_read(adcPin);
					
					// Disable ADC conversions and turn off ADC interrupt
					cbi(ADCSRA, ADEN);
					cbi(ADCSRA, ADIE);
				}
				
				PORTA &= ~(1<<ledPin); // Turn off LED
				sbi(PCMSK0, PCINT6); // Re-enable button interrupt
				eepromBuffer[eepromBufferCounter++] = (loggingResult >> 8);
				eepromBuffer[eepromBufferCounter++] = (loggingResult & 0xFF);
				
				// Once our 32 byte buffer is full, write it to the I2C EEPROM
				if (eepromBufferCounter >= EEPROM_BUFFER_LENGTH) {
					soft_i2c_eeprom_page_write(EEPROM_ADDR, eepromPageCounter, eepromPageByteCounter, eepromBuffer, EEPROM_BUFFER_LENGTH);
					eepromPageByteCounter += eepromBufferCounter;
					eepromBufferCounter = 0;
					
					// If we have almost filled up the current page (224 out of 256 bytes) and we aren't on the last page of the EEPROM (511)
					// then blank the next page 
					if (eepromPageByteCounter == 224 && eepromPageCounter < 511) {
						watchdog_sleep(T16MS); // Wait for page write from before to finish
						soft_i2c_eeprom_page_blank(EEPROM_ADDR, eepromPageCounter+1);
					}
					
					// Increment page address
					if (eepromPageByteCounter == 0) { 
						if (eepromPageCounter >= 511) { // EEPROM full, sleep until USB plugged in or button pressed again
							logging = false;
						}
						eepromPageCounter++;
						eepromPageByteCounter = 0;
					}
				}
				
				// Wait delay time
				uint32_t delayCount = delaySeconds + (delayMinutes * 60) + (delayHours * 3600) + (delayDays * 86400);
				while (delayCount != 0 && logging == true) {
					rtc_sleep_1_sec();
					delayCount--;
					
					// Trim RTC based on temperature when required
					autoTrimIntervalCounter--;
					if (autoTrimIntervalCounter == 0) {
						autoTrimIntervalCounter = autoTrimInterval;
						
						int tempResult = tmp102_read_temp(TEMP_ADDR);
						set_trim_from_temp(tempResult);
					}
				}
			}
			
			// Button was held to exit logging, if any data remains in the buffer, write to EEPROM
			if (eepromBufferCounter >= 1) {
				soft_i2c_eeprom_page_write(EEPROM_ADDR, eepromPageCounter, eepromPageByteCounter, eepromBuffer, eepromBufferCounter);
			}
			
			// Enable pin change on 3.3V LDO sense and disable RTC 1Hz sense
			sbi(PCMSK0, PCINT2);
			cbi(PCMSK0, PCINT3);
		}
		
		else { // Sleep and/or trim RTC from temperature if auto trim is on
			// Check if RTC is running
			soft_i2c_master_start_with_8bit_address(RTC_ADDRESS, RTC_RTCSEC, REPEATSTART);
			uint8_t isRtcRunning = soft_i2c_master_read(1);
			soft_i2c_master_stop();
			
			// If RTC is running and rtcAutoTrim is set, wake up according to rtc trim interval and adjust the trim based on temperature
			if ((isRtcRunning & RTC_RTCSEC_ST_BIT) && rtcAutoTrim == true) {
				// Read RTC auto trim interval time
				uint8_t autoTrimIntervalSeconds = eeprom_read_byte((uint8_t*) autoTrimIntervalLocation);
				uint8_t autoTrimIntervalMinutes = eeprom_read_byte((uint8_t*) autoTrimIntervalLocation+1);
				uint8_t autoTrimIntervalHours = eeprom_read_byte((uint8_t*) autoTrimIntervalLocation+2);
				uint8_t autoTrimIntervalDays = eeprom_read_byte((uint8_t*) autoTrimIntervalLocation+3);
				
				// Unprogrammed rtc auto trim interval time?
				if (autoTrimIntervalSeconds == blankEeprombyte) {
					autoTrimIntervalMinutes = 0;
					autoTrimIntervalMinutes = 30; // Set 30 minutes as the default
					autoTrimIntervalHours = 0;
					autoTrimIntervalDays = 0;
				}
				
				// Set clock output at 1Hz
				soft_i2c_master_start_with_8bit_address(RTC_ADDRESS, RTC_CONTROL, NOREPEATSTART);
				soft_i2c_master_write(RTC_CONTROL_SQWEN_BIT);
				soft_i2c_master_stop();
				
				// Enable RTC 1Hz sense
				sbi(PCMSK0, PCINT3);
				
				// While we aren't logging and the USB isn't plugged in
				while (logging == false && (!(PINA & (1<<usbSensePin)))) {
					int tempResult = tmp102_read_temp(TEMP_ADDR);
					set_trim_from_temp(tempResult);
					
					// Wait the trim interval
					uint32_t autoTrimInterval = autoTrimIntervalSeconds + (autoTrimIntervalMinutes * 60) + (autoTrimIntervalHours * 3600) + (autoTrimIntervalDays * 86400);
					while (autoTrimInterval != 0 && logging == false && (!(PINA & (1<<usbSensePin)))) {
						rtc_sleep_1_sec();
						autoTrimInterval--;
					}
				}
				
				// Disable RTC 1Hz sense
				cbi(PCMSK0, PCINT3);
			}
			else {
				system_sleep();
			}
		}
	}
	
	return 0;
}

EMPTY_INTERRUPT(WDT_vect);
EMPTY_INTERRUPT(ADC_vect);

ISR(PCINT0_vect) {
	// Check if button is being held down for 2 seconds, if it is, switch the state of logging variable
	uint8_t buttonHeldCounter = 0;
	while (PINA & (1<<buttonPin)) {
		_delay_ms(20); // Multiply by 8 for real delay ms
		buttonHeldCounter++;
		
		if (buttonHeldCounter >= 12) { // 2 seconds
			if (logging == false) {
				logging = true;
			}
			else {
				logging = false;
			}
			PORTA |= (1<<ledPin); // Turn on LED
			_delay_ms(50); // Multiply by 8 for real delay ms
			PORTA &= ~(1<<ledPin); // Turn off
			break;
		}
	}
	
	// Wait for button release (so we don't generate another interrupt when released)
	while (PINA & (1<<buttonPin)) {
		_delay_ms(20); // Multiply by 8 for real delay ms
	}
}
